package chap7;

import java.sql.Date;
import java.sql.Statement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;

public class SQParameterSample3 {
	public static void main(String[] args) {
		Connection con = null;
		Statement st = null;
		PreparedStatement pst = null;

		try {
			// ドライバの登録
			Class.forName("oracle.jdbc.driver.OracleDriver");

			// 接続処理
			con =
				DriverManager.getConnection(
						"jdbc:oracle:thin:@localhost:1521:xe","scott", "tiger");
			pst =
				con.prepareStatement("UPDATE user_list SET gdate = ?");

			// 現在時刻
			Calendar cal = Calendar.getInstance();
			pst.setDate(1, new Date(cal.getTimeInMillis()), cal);
			System.out.printf("%dレコード更新(setDate)\n",pst.executeUpdate());

			st = con.createStatement();
			ResultSet rs = st.executeQuery("SELECT * FROM user_list");
			while(rs.next()){
				System.out.println(rs.getInt("id") + " " + rs.getString("gdate"));
			}

			// 5日前に設定
			cal.add(Calendar.DATE, -5);
			pst.setTime(1, new Time(cal.getTimeInMillis()), cal);
			System.out.printf("%dレコード更新(setTime)\n",pst.executeUpdate());

			st = con.createStatement();
			rs = st.executeQuery("SELECT * FROM user_list");
			while(rs.next()){
				System.out.println(rs.getInt("id") + " " + rs.getString("gdate"));
			}

			// 日付、時刻の設定
			pst.setTimestamp(1, new Timestamp(cal.getTimeInMillis()), cal);
			System.out.printf("%dレコード更新(setTimestamp)\n",pst.executeUpdate());

			st = con.createStatement();
			rs = st.executeQuery("SELECT * FROM user_list");
			while(rs.next()){
				System.out.println(rs.getInt("id") + " " + rs.getString("gdate"));
			}
		}
		catch (ClassNotFoundException e) {
			System.out.println("登録に失敗しました; "+e.getMessage());
		}
		catch (SQLException e) {
			System.out.println("処理に失敗しました; "+e.getMessage());
		}
		finally {
			try {
				if (st!=null ) st.close();
			} catch (Exception e) {
			}
			try {
				if (pst!=null ) pst.close();
			} catch (Exception e) {
			}
			try {
				// 切断処理
				if (con!=null ) con.close();
			} catch (Exception e) {
			}
		}
	}
}